@@ -15,20 +15,11 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-__version__ = '1.3.1'
-__author__ = 'Carlos Perez, Carlos_Perez@darkoperator.com'
-__name__ = 'cli'
-
-__doc__ = """
-DNSRecon https://www.darkoperator.com
-
- by Carlos Perez, Darkoperator
-"""
-
 import datetime
 import json
 import os
+import re
+import socket
 import sqlite3
 import sys
 from argparse import ArgumentError, ArgumentParser, RawTextHelpFormatter
@@ -48,8 +39,8 @@ import dns.rdatatype
 import dns.resolver
 import dns.reversename
 import dns.zone
+import httpx
 import netaddr
-import requests
 from dns.dnssec import algorithm_to_text
 from loguru import logger
 
@@ -59,6 +50,17 @@ from dnsrecon.lib.dnshelper import DnsHelper
 from dnsrecon.lib.whois import *
 from dnsrecon.lib.yandexenum import *
 
+__version__ = '1.5.3'
+__author__ = 'Carlos Perez, Carlos_Perez@darkoperator.com'
+__name__ = 'cli'
+
+__doc__ = """
+DNSRecon https://www.darkoperator.com
+
+ by Carlos Perez, Darkoperator
+"""
+
+
 # Global Variables for Brute force Threads
 brtdata = []
 
@@ -210,11 +212,11 @@ def check_nxdomain_hijack(nameserver):
         try:
             answers = res.resolve(testname, record_type, tcp=True)
         except (
+            OSError,
             dns.resolver.NoNameservers,
             dns.resolver.NXDOMAIN,
             dns.exception.Timeout,
             dns.resolver.NoAnswer,
-            socket.error,
             dns.query.BadResponse,
         ):
             continue
@@ -248,8 +250,8 @@ def brute_tlds(res, domain, verbose=False, thread_num=None):
 
     total_tlds = []
     try:
-        tlds_list = requests.get(
-            'https://raw.githubusercontent.com/publicsuffix/list/master/public_suffix_list.dat', timeout=30
+        tlds_list = httpx.get(
+            'https://raw.githubusercontent.com/publicsuffix/list/master/public_suffix_list.dat', timeout=30.0
         ).text
     except Exception as e:
         tlds_list = ''
@@ -485,6 +487,13 @@ def brute_domain(
 
     found_hosts = []
 
+    # Define a safe root directory for dictionary files
+    safe_root = os.path.join(os.path.dirname(__file__), 'data')
+    dictfile = os.path.normpath(os.path.join(safe_root, dictfile))
+    if not dictfile.startswith(safe_root):
+        logger.error('Invalid dictionary file path.')
+        return None
+
     # Check if the Dictionary file exists
     if os.path.isfile(dictfile):
         with open(dictfile) as fd:
@@ -559,7 +568,7 @@ def in_cache(res, dict_file, ns):
     return found_records
 
 
-def se_result_process(res, se_entries):
+def se_result_process(res, domain, se_entries):
     """
     This function processes the results returned from a Search Engine and does
     an A and AAAA query for the IP of the found host. Prints and returns a dictionary
@@ -575,7 +584,7 @@ def se_result_process(res, se_entries):
                 continue
 
             logger.info(f'\t {type_} {name_} {address_or_target_}')
-            resolved_se_entry = {'type': type_, 'name': name_}
+            resolved_se_entry = {'type': type_, 'name': name_, 'domain': domain}
 
             if type_ == 'A':
                 resolved_se_entry['address'] = address_or_target_
@@ -649,7 +658,7 @@ def whois_ips(res, ip_list):
         logger.info('The following IP Ranges were found:')
         for i in range(len(list_whois)):
             logger.info(
-                '\t {0} {1}-{2} {3}'.format(
+                '\t {} {}-{} {}'.format(
                     str(i) + ')',
                     list_whois[i]['start'],
                     list_whois[i]['end'],
@@ -663,7 +672,7 @@ def whois_ips(res, ip_list):
 
         if 'a' in answer:
             for i in range(len(list_whois)):
-                logger.info('Performing Reverse Lookup of range {0}-{1}'.format(list_whois[i]['start'], list_whois[i]['end']))
+                logger.info('Performing Reverse Lookup of range {}-{}'.format(list_whois[i]['start'], list_whois[i]['end']))
                 found_records.append(brute_reverse(res, expand_range(list_whois[i]['start'], list_whois[i]['end'])))
 
         elif 'n' in answer:
@@ -672,7 +681,7 @@ def whois_ips(res, ip_list):
             for a in answer:
                 net_selected = list_whois[int(a)]
                 logger.info(net_selected['orgname'])
-                logger.info('Performing Reverse Lookup of range {0}-{1}'.format(net_selected['start'], net_selected['end']))
+                logger.info('Performing Reverse Lookup of range {}-{}'.format(net_selected['start'], net_selected['end']))
                 found_records.append(brute_reverse(res, expand_range(net_selected['start'], net_selected['end'])))
     else:
         logger.error('No IP Ranges were found in the Whois query results')
@@ -842,7 +851,7 @@ def write_db(db, data):
         if re.match(r'PTR|^[A]$|AAAA', n['type']):
             query = (
                 'insert into data( domain, type, name, address ) '
-                + 'values( "{domain}", "{type}", "{name}","{address}" )'.format(**n)
+                + 'values( "{domain}", "{type}", "{name}", "{address}" )'.format(**n)
             )
 
         elif re.match(r'NS$', n['type']):
@@ -864,21 +873,27 @@ def write_db(db, data):
             )
 
         elif re.match(r'TXT', n['type']):
-            query = 'insert into data( domain, type, text) ' + 'values( "{domain}", "{type}","{strings}" )'.format(**n)
+            query = 'insert into data( domain, type, text) ' + 'values( "{domain}", "{type}", "{strings}" )'.format(**n)
 
         elif re.match(r'SPF', n['type']):
-            query = 'insert into data( domain, type, text) ' + 'values( "{domain}", "{type}","{strings}" )'.format(**n)
+            query = 'insert into data( domain, type, text) ' + 'values( "{domain}", "{type}", "{strings}" )'.format(**n)
 
         elif re.match(r'SRV', n['type']):
             query = (
                 'insert into data( domain, type, name, target, address, port ) '
-                + 'values( "{domain}", "{type}", "{name}" , "{target}", "{address}" ,"{port}" )'.format(**n)
+                + 'values( "{domain}", "{type}", "{name}", "{target}", "{address}", "{port}" )'.format(**n)
             )
 
         elif re.match(r'CNAME', n['type']):
             query = (
                 'insert into data( domain, type, name, target ) '
-                + 'values( "{domain}", "{type}", "{name}" , "{target}" )'.format(**n)
+                + 'values( "{domain}", "{type}", "{name}", "{target}" )'.format(**n)
+            )
+        elif re.match(r'CAA', n['type']):
+            # Preserve some of the previous behavior when writing CAA records to the database would fall into the else clause
+            text = ','.join([f'{key}={value}' for key, value in n.items() if key != 'type'])
+            query = "insert into data( domain, type, name, target, address, text ) values ('{domain}', '{type}', '{name}', '{target}', '{address}', '{text}')".format(
+                **n, text=text
             )
 
         else:
@@ -887,7 +902,7 @@ def write_db(db, data):
             del n['type']
             record_data = ''.join([f'{key}={value},' for key, value in n.items()])
             records = [t, record_data]
-            query = 'insert into data(domain,type,text) values ("%(domain)", \'' + records[0] + "','" + records[1] + "')"
+            query = 'insert into data( domain, type, text ) values ("%(domain)", \'' + records[0] + "', '" + records[1] + "')"
 
         # Execute Query and commit
         cur.execute(query)
@@ -941,20 +956,15 @@ def check_bindversion(res, ns_server, timeout):
     version = ''
 
     if not CONFIG or not CONFIG.get('disable_check_bindversion', False):
-        request = dns.message.make_query('version.bind', 'txt', 'ch')
+        flags = dns.flags.RD if res._recursion_desired else 0
+        request = dns.message.make_query('version.bind', 'txt', 'ch', flags=flags)
         try:
             response = res.query(request, ns_server, timeout=timeout, one_rr_per_rrset=True)
             if len(response.answer) > 0:
                 version = response.answer[0].to_text().split(' ')[-1]
                 logger.info(f'\t Bind Version for {ns_server} {version}')
 
-        except (
-            dns.resolver.NXDOMAIN,
-            dns.exception.Timeout,
-            dns.resolver.NoAnswer,
-            socket.error,
-            dns.query.BadResponse,
-        ):
+        except (OSError, dns.resolver.NXDOMAIN, dns.exception.Timeout, dns.resolver.NoAnswer, dns.query.BadResponse):
             pass
 
     return version
@@ -976,7 +986,7 @@ def check_recursive(res, ns_server, timeout):
             if result:
                 logger.error(f'\t Recursion enabled on NS Server {ns_server}')
             is_recursive = True
-        except (socket.error, dns.exception.Timeout):
+        except (OSError, dns.exception.Timeout):
             pass
 
     return is_recursive
@@ -1187,7 +1197,7 @@ def general_enum(
         # Do Bing Search enumeration if selected
         if do_bing:
             logger.info('Performing Bing Search Enumeration')
-            bing_rcd = se_result_process(res, scrape_bing(domain))
+            bing_rcd = se_result_process(res, domain, scrape_bing(domain))
             if bing_rcd:
                 for r in bing_rcd:
                     if 'address' in bing_rcd:
@@ -1197,7 +1207,7 @@ def general_enum(
         # Do Yandex Search enumeration if selected
         if do_yandex:
             logger.info('Performing Yandex Search Enumeration')
-            yandex_rcd = se_result_process(res, scrape_bing(domain))
+            yandex_rcd = se_result_process(res, domain, scrape_bing(domain))
             if yandex_rcd:
                 for r in yandex_rcd:
                     if 'address' in yandex_rcd:
@@ -1206,7 +1216,7 @@ def general_enum(
 
         if do_crt:
             logger.info('Performing Crt.sh Search Enumeration')
-            crt_rcd = se_result_process(res, scrape_crtsh(domain))
+            crt_rcd = se_result_process(res, domain, scrape_crtsh(domain))
             if crt_rcd:
                 for r in crt_rcd:
                     if 'address' in crt_rcd:
@@ -1234,7 +1244,8 @@ def query_ds(res, target, ns, timeout=5.0):
     happens.
     """
     try:
-        query = dns.message.make_query(target, dns.rdatatype.DS, dns.rdataclass.IN)
+        flags = dns.flags.RD if res._recursion_desired else 0
+        query = dns.message.make_query(target, dns.rdatatype.DS, dns.rdataclass.IN, flags=flags)
         query.flags += dns.flags.CD
         query.use_edns(edns=True, payload=4096)
         query.want_dnssec(True)
@@ -1254,7 +1265,7 @@ def get_constants(prefix):
     """
     Create a dictionary mapping socket module constants to their names.
     """
-    return dict((getattr(socket, n), n) for n in dir(socket) if n.startswith(prefix))
+    return {getattr(socket, n): n for n in dir(socket) if n.startswith(prefix)}
 
 
 def socket_resolv(target):
@@ -1281,14 +1292,14 @@ def lookup_next(target, res):
     """
     Try to get the most accurate information for the record found.
     """
-    DnsHelper(target)
+    DnsHelper(target, recursion_desired=res._recursion_desired)
     returned_records = []
 
     if re.search(r'^_[A-Za-z0-9_-]*._[A-Za-z0-9_-]*.', target, re.I):
         srv_answer = res.get_srv(target)
         if len(srv_answer) > 0:
             for r in srv_answer:
-                logger.info('\t {0}'.format(' '.join(r)))
+                logger.info('\t {}'.format(' '.join(r)))
                 returned_records.append(
                     {
                         'type': r[0],
@@ -1303,13 +1314,13 @@ def lookup_next(target, res):
         txt_answer = res.get_txt(target)
         if len(txt_answer) > 0:
             for r in txt_answer:
-                logger.info('\t {0}'.format(' '.join(r)))
+                logger.info('\t {}'.format(' '.join(r)))
                 returned_records.append({'type': r[0], 'name': r[1], 'strings': r[2]})
         else:
             txt_answer = res.get_txt(target)
             if len(txt_answer) > 0:
                 for r in txt_answer:
-                    logger.info('\t {0}'.format(' '.join(r)))
+                    logger.info('\t {}'.format(' '.join(r)))
                     returned_records.append({'type': r[0], 'name': r[1], 'strings': r[2]})
             else:
                 logger.info(f'\t A {target} no_ip')
@@ -1338,7 +1349,8 @@ def lookup_next(target, res):
 
 
 def get_a_answer(res, target, ns, timeout):
-    query = dns.message.make_query(target, dns.rdatatype.A, dns.rdataclass.IN)
+    flags = dns.flags.RD if res._recursion_desired else 0
+    query = dns.message.make_query(target, dns.rdatatype.A, dns.rdataclass.IN, flags=flags)
     query.flags += dns.flags.CD
     query.use_edns(edns=True, payload=4096)
     query.want_dnssec(True)
@@ -1356,7 +1368,7 @@ def get_next(res, target, ns, timeout):
     return next_host
 
 
-def ds_zone_walk(res, domain, lifetime):
+def ds_zone_walk(res, domain, request_timeout):
     """
     Perform DNSSEC Zone Walk using NSEC records found in the error an additional
     records section of the message to find the next host to query in the zone.
@@ -1374,7 +1386,7 @@ def ds_zone_walk(res, domain, lifetime):
                 nameserver = first_ns[2]
                 if nameserver:
                     logger.info(f'Name Server {nameserver} will be used')
-                    res = DnsHelper(domain, nameserver, lifetime)
+                    res = DnsHelper(domain, nameserver, request_timeout, recursion_desired=res._recursion_desired)
 
         if not nameserver:
             logger.error('This zone appears to be misconfigured, no SOA record found.')
@@ -1472,7 +1484,7 @@ def ds_zone_walk(res, domain, lifetime):
     except EOFError:
         logger.error(f'SoA nameserver {nameserver} failed to answer the DNSSEC query for {target}')
 
-    except socket.error:
+    except OSError:
         logger.error(f'SoA nameserver {nameserver} failed to answer the DNSSEC query for {domain}')
 
     # Give a summary of the walk
@@ -1632,6 +1644,11 @@ def main():
             help='Disables check for recursion on name servers',
             action='store_true',
         )
+        parser.add_argument(
+            '--disable_recurs',
+            help='Disable recursion desired flag in queries.',
+            action='store_true',
+        )
         parser.add_argument(
             '--disable_check_bindversion',
             help='Disables check for BIND version on name servers',
@@ -1654,6 +1671,7 @@ Possible types:
     bing:     Perform Bing search for subdomains and hosts.
     yand:     Perform Yandex search for subdomains and hosts.
     crt:      Perform crt.sh search for subdomains and hosts.
+    caa:      CAA records.
     snoop:    Perform cache snooping against all NS servers for a given domain, testing
               all with file containing the domains, file given with -D option.
 
@@ -1694,6 +1712,7 @@ Possible types:
         'bing': {'domain': True, 'dictionary': False},
         'yand': {'domain': True, 'dictionary': False},
         'crt': {'domain': True, 'dictionary': False},
+        'caa': {'domain': True, 'dictionary': False},
         'rvl': {'domain': False, 'dictionary': False},
         'zonewalk': {'domain': True, 'dictionary': False},
         'brt': {'domain': True, 'dictionary': True},
@@ -1848,7 +1867,7 @@ Possible types:
         logger.info(f'Starting enumeration for domain: {domain}')
 
         # Initialize the resolver for the current domain
-        res = DnsHelper(domain, ns_server, request_timeout, proto)
+        res = DnsHelper(domain, ns_server, request_timeout, proto, recursion_desired=not arguments.disable_recurs)
 
         scan_info = [' '.join(sys.argv), str(datetime.datetime.now())]
 
@@ -1921,24 +1940,43 @@ Possible types:
 
                 elif type_ == 'bing':
                     logger.info(f'{type_}: Performing Bing Search Enumeration against {domain}...')
-                    bing_enum_records = se_result_process(res, scrape_bing(domain))
+                    bing_enum_records = se_result_process(res, domain, scrape_bing(domain))
                     if bing_enum_records is not None and do_output:
                         all_returned_records.extend(bing_enum_records)
 
                 elif type_ == 'yand':
                     logger.info(f'{type_}: Performing Yandex Search Enumeration against {domain}...')
-                    yandex_enum_records = se_result_process(res, scrape_yandex(domain))
+                    yandex_enum_records = se_result_process(res, domain, scrape_yandex(domain))
                     if yandex_enum_records is not None and do_output:
                         all_returned_records.extend(yandex_enum_records)
 
                 elif type_ == 'crt':
                     logger.info(f'{type_}: Performing Crt.sh Search Enumeration against {domain}...')
-                    crt_enum_records = se_result_process(res, scrape_crtsh(domain))
+                    crt_enum_records = se_result_process(res, domain, scrape_crtsh(domain))
                     if crt_enum_records is not None and do_output:
                         all_returned_records.extend(crt_enum_records)
                     else:
                         logger.info('[-] No records returned from crt.sh enumeration')
 
+                elif type_ == 'caa':
+                    logger.info(f'{type_}: Performing CAA Record Enumeration against {domain}...')
+                    caa_records = res.get_caa()
+                    if caa_records:
+                        for record in caa_records:
+                            record_type, name, value = record[:3]
+                            logger.info(f'     {record_type} {name} {value}')
+                        if do_output:
+                            # Convert to the format expected by output functions
+                            caa_enum_records = []
+                            for record in caa_records:
+                                record_type, name, value = record[:3]
+                                caa_enum_records.append(
+                                    {'type': record_type, 'name': name, 'address': value, 'target': name, 'domain': domain}
+                                )
+                            all_returned_records.extend(caa_enum_records)
+                    else:
+                        logger.info(f'{type_}: No CAA records found for {domain}')
+
                 elif type_ == 'snoop':
                     if not (dictionary and ns_server):
                         logger.error(f'{type_}: A dictionary file and at least one Name Server have to be specified!')
